------------------------------------------------------------------------------
-- GUI for showing the result of observing objects in Curry programs.
------------------------------------------------------------------------------

import Observe(clearLogFile)
import CoosyShowObserve(readAndPrintEvents,ViewConf(..))
import CoosyTrace(logFileClear)
import GUI
import System

------------------------------------------------------------------------------
-- A GUI for the translator:

main = runGUI "COOSY" addline_gui

addline_gui =
 Col [] [
   Label [Text "Curry Object Observation System",
            Background "red", FillX],
   Row [CenterAlign]
    [Label [FillX],
     Button clearTrace [Text "Clear"],
     Button showObserve [Text "Show"],
     Button exitGUI [Text "Exit"],
     Label [Width 10],
     CheckButton [Text "show bound logical variables",
                  CheckInit "1",
                  WRef logVarCheck,
                  Cmd showObserve],
     Label [FillX],
     MenuButton
       [Text "Infos...",
        Menu [MButton help  "How to use COOSY",
              MButton about "About COOSY"]]],
   TextEditScroll [WRef rtxt, Height 40, Text initMsg, Background "white"],
   Label [WRef status, Text "Status: waiting", Background "green", FillX]
   ]
 where
   rtxt,status,logVarCheck free

   clearTrace wp = do
     clearLogFile
     writeFile logFileClear "1"
     setValue rtxt ("Logfiles cleared.") wp

   showObserve wp = do
     setValue status "Status: running" wp
     setConfig status (Background "red") wp
     setValue rtxt "" wp
     logVarSel <- getValue logVarCheck wp
     catchFail
        (readAndPrintEvents (\s -> appendValues rtxt wp s)
                            (toViewConf logVarSel))
        (appendValue rtxt failMsg wp)
     setValue status "Status: waiting" wp
     setConfig status (Background "green") wp

   help wp = do
     dir <- getHomeDir
     helptext <- readFile (dir++"Help.txt")
     setValue rtxt helptext wp
     return []

   about wp = do
     dir <- getHomeDir
     helptext <- readFile (dir++"README.txt")
     setValue rtxt helptext wp
     return []

appendValues _ _ [] = done
appendValues rtxt wp (s:ss) 
  = if elem (chr 7) (s:ss) then appendGray rtxt wp (s:ss)
      else appendTaggedValue rtxt (s:ss) "black" wp 

appendGray _ _ [] = done
appendGray rtxt wp (s:ss) 
  = appendTaggedValue rtxt gray "gray" wp >> appendBlack rtxt wp rest
  where
    (gray,rest) = span (/= (chr 7)) (s:ss)

appendBlack _ _ [] = done
appendBlack rtxt wp (_:ss) = appendValue rtxt black wp 
                             >> appendGray rtxt wp rest
  where
    (black,_:rest) = span (/= (chr 7)) ss

toViewConf :: String -> ViewConf
toViewConf "1" = ShowLogVarBinds
toViewConf "0" = HideLogVarBinds

condIO :: Bool -> IO () -> IO ()
condIO True a = a
condIO False _ = return ()


getHomeDir = do
  coosyhome <- getEnviron "COOSYHOME"
  return $ if coosyhome=="" then "" else coosyhome++"/"

initMsg =
   "IMPORTANT NOTE:\n\n" ++
   "Don't forget to press 'clear' before you observe a new program execution!"

failMsg =
   "Failure occurred during reading of trace file!\n\n"++
   "Press 'clear' button and run again your program."

------------------------------------------------------------------------------
